######################################################################################
# Compatible with GeneEvolve version 0.69 +                                          #
# SCRIPT FOR GENERATING GRAPHICS FROM PEDEVOLVE SIMULATIONS                          #
# by Matthew C Keller, updated 11/18/2007                                            #
######################################################################################




############################################################################
#GENERATE.GRAPHICS
ge.graphics <- function(date.names,PARAMS,VARIANCES,MX.info=NULL,PE.Var2,PE.Var3,rel.correlations,track.changes,data.info){

#Open PDF document
name <- paste("GE_Graph-",date.names,".pdf",sep="")
pdf(name,width=11, height=8.5,pointsize=12, paper='special')
############################################################################





############################################################################
#1 OUTPUT BASIC INFORMATION ABOUT RUN

line.nums <- 25
op <- par(mar=c(2,2,line.nums,1),xpd=TRUE,oma=c(1,1,1,1))

vtype <- "vertical transmission from parental phenotypes to offspring"
if (PARAMS$am.model == "I"){amtype <- "primary phenotypic assortment - correlation b/w mates due to their choosing similar phenotypes"}
if (PARAMS$am.model == "II"){amtype <- "social homogamy - mating similarity comes through correlated environmental factors"}
if (PARAMS$am.model == "III"){amtype <- "familial social homogamy - mating similarity comes only through correlated F (familial) factor"}
input <- unlist(VARIANCES)[1:PARAMS$number.parameters]
data.input <- c(data.info$n.mzm+data.info$n.mzf,data.info$n.dzm+data.info$n.dzf+data.info$n.dzos,data.info$n.data[2:5])

plot(1:14,1:14,type="n",axes=FALSE,bty='n',)
mtext(paste("Results Summary from GeneEvolve Simulation ",PARAMS$t4),line=line.nums-1,cex=1.5)

mtext("Basic Parameters:",line=line.nums-3,cex=1,at=1,adj=0)
mtext(paste("Working Directory:",as.character(PARAMS$working.directory)),line=line.nums-4,cex=.8,adj=0,at=1.25)
mtext(paste("Number of generations:",as.character(PARAMS$number.generations)),line=line.nums-5,cex=.8,adj=0,at=1.25)
mtext(paste("Population size at start:",as.character(PARAMS$start.pop.size)),line=line.nums-6,cex=.8,adj=0,at=1.25)
mtext(paste("Number of genes:",as.character(PARAMS$number.genes)),line=line.nums-7,cex=.8,adj=0,at=1.25)
mtext(paste("Vertical Trans. model:",as.character(vtype)),line=line.nums-8,cex=.8,adj=0,at=1.25)
mtext(paste("Assort. Mating model:",as.character(amtype)," AM = ",as.character(VARIANCES$latent.AM[1])),line=line.nums-9,cex=.8,adj=0,at=1.25)

mtext("Sample Sizes in Dataset:",line=line.nums-11,cex=1,at=1,adj=0)
mtext(c("MZ","DZ","Parents","Sibs","Spouses","Children"),line=line.nums-12, cex=.7,at=c(2,4,6,8,10,12),adj=0.5)
mtext(data.input,line=line.nums-13, cex=.7,at=c(2,4,6,8,10,12),adj=0.5)

mtext(c("Variance Components - User Input:","(Note: U+MZ+TW=E)"),line=line.nums-15,cex=c(1,.7),at=c(1,5),adj=0)
mtext(names(input),line=line.nums-16, cex=.7,at=(1:14),adj=0.5)
mtext(input,line=line.nums-17, cex=.7,at=(1:14),adj=0.5)

mtext("Time",line=line.nums-19,cex=1,adj=0,at=1)
mtext(paste("Simulation started: ",as.character(PARAMS$t1)),line=line.nums-20,at=1.25,cex=.8,adj=0)
mtext(paste("Simulation ended: ",as.character(Sys.time())),line=line.nums-21,at=1.25,cex=.8,adj=0)
mtext(paste("Minutes taken - looping through generations: ",round(difftime(PARAMS$t3,PARAMS$t1,units='mins'),2)),line=line.nums-22,at=1.25,cex=.8,adj=0)
mtext(paste("Minutes taken - creating pedigree datasets: ",round(difftime(PARAMS$t4,PARAMS$t2,units='mins'),2)),line=line.nums-23,at=1.25,cex=.8,adj=0)
mtext(paste("Minutes taken - finding relative correlations: ",round(difftime(PARAMS$t5,PARAMS$t4,units='mins'),2)),line=line.nums-24,at=1.25,cex=.8,adj=0)
mtext(paste("Minutes taken - TOTAL: ",round(difftime(Sys.time(),PARAMS$t1,units='mins'),2)),line=line.nums-25,at=1.25,cex=.8,adj=0)

mtext("Warnings in script:",line=line.nums-27,cex=1,adj=0,at=1)
mtext(PARAMS$startpop.warn,line=line.nums-28,at=1.25,cex=.8,adj=0)
if (PARAMS$run.cascade.model=="yes") {
  mtext(paste("Cascade:", MX.info$CODE21),line=line.nums-29,at=1.25,cex=.8,adj=0)
  if (PARAMS$run.stealth.model=="yes") {mtext(paste("Stealth:", MX.info$CODE1),line=line.nums-30,at=1.25,cex=.8,adj=0)}}


if (max(PE.Var3)>10) {
  mtext("The variance of a component is greater than 10x its starting value. Graphing will fail. Your starting values have created an unstable model.",
        line=line.nums-32,at=1.25,cex=.8,adj=0)}

############################################################################






###############################################################################
#2 GRAPHING PEDIGREE CORRELATIONS
par(mar=c(3,2,6,1),xpd=TRUE,oma=c(2,2,2,2))

locs <-rel.correlations + .03
locs[locs<0] <- .05

mycolors <- c('darkblue','mediumpurple','turquoise','goldenrod4','lightgoldenrod',
              'brown','orangered','darkgreen','darkseagreen','snow2')
       
bpl <- barplot(rel.correlations, col=mycolors,ylim=c(-.1,1), xlab="Relative Type",ylab="Correlation",
        names.arg=c("MZ","DZ","Sib","Par.Cd","Gp.Gcd","MZAv","Avnc","MZcs","Cs","Sps"),cex.names=1)
text(bpl,locs,round(rel.correlations,2))

varcovar <- c(PE.Var2[c(17:30,33)],PE.Var2[31]*track.changes["cor.spouses",ncol(track.changes)])/PE.Var2[31]
lim <- par("usr")
labels.at <- seq(lim[1],lim[2],length.out=length(varcovar))

mtext("Correlations between 10 Relative Types",line=5,cex=1.5)
mtext("True Standardized Variance Components in Dataset",line=3,cex=.75)
mtext(c(names(input),"Covs","AM"),line=2, cex=.7,at=labels.at,adj=0.5)
mtext(round(varcovar,2),line=1, cex=.7,at=labels.at,adj=0.5)
################################################################################




try({



############################################################################
#3 GRAPHING TRACK CHANGES

#Axis break stuff
ymax <- max(ceiling(PE.Var3[,1:(ncol(PE.Var3)-1)]*11))/10

#make new track.changes that includes "E"
par(mar=c(4,4,8,1),oma=c(1,1,1,1),xpd=NA)
names2 <- rownames(PE.Var3)
start <- PE.Var3[names2,1]
end <- PE.Var3[names2,(ncol(PE.Var3)-1)]
dat <- PE.Var3[names2,ncol(PE.Var3)]
names3 <- names2[1:nrow(PE.Var3)-1]        #all names except r(sps) - meaning that we won't graph r(sps)
names3 <- names3[PE.Var3[1:(nrow(PE.Var3)-1),ncol(PE.Var3)] != 0]  
use.colors <- c(c('darkblue','red','green','orange','yellow','purple','brown','cyan4','olivedrab4','hotpink1','lightblue')[1:(length(names3)-1)],"black")

#Plot it
plot(PE.Var3["V(P)",1:(ncol(PE.Var3)-1)],ylim=c(0,ymax),type='n',lwd=2,col='black',
     xlab='Generation Number',ylab='Variance or Covariance',bty='n',axes=TRUE)
lim <- par("usr")
labels.at <- seq(lim[1],lim[2],length.out=length(names2)+1)

mtext("Change in Variance Across Generations - Includes V(P)",line=6,cex=1.5)
mtext("Unstandardized Variance Components",line=5,cex=.7)
mtext(names2,side=3,outer=FALSE,line=4, cex=.7,at=labels.at[2:length(labels.at)])
mtext(c("Start:",round(start,2)),side=3,outer=FALSE,line=3, cex=.7,at=labels.at[1:length(labels.at)])
mtext(c("End:",round(end,2)),side=3,outer=FALSE,line=2, cex=.7,at=labels.at[1:length(labels.at)])
mtext(c("Data:",round(dat,2)),side=3,outer=FALSE,line=1, cex=.7,at=labels.at[1:length(labels.at)])

legend(x=c(lim[1],(lim[2]-.1*(lim[2]-lim[1]))),
       y=c(lim[4],(lim[4]-.15*(lim[4]-lim[3]))),
       legend=names3,lty=1,col=use.colors,lwd=2,ncol=min(5,length(names3)),bg='snow2',yjust=1,text.width=1)

for (i in 1:length(names3)){
lines(PE.Var3[names3[i],1:(ncol(PE.Var3)-1)],col=use.colors[i],lwd=2)}
###############################################################################








############################################################################
#4 GRAPHING TRACK CHANGES

PE.Var4 <- PE.Var3[- (nrow(PE.Var3)-1),]      #remove V(P), which is next to last row

#Axis break stuff
ymax <- max(ceiling(PE.Var4[,1:(ncol(PE.Var4)-1)]*11))/10

#make new track.changes that includes "E"
par(mar=c(4,4,8,1),oma=c(1,1,1,1),xpd=NA)
names5 <- rownames(PE.Var4)
start2 <- PE.Var4[names5,1]
end2 <- PE.Var4[names5,(ncol(PE.Var4)-1)]
dat2 <- PE.Var4[names5,ncol(PE.Var4)]
names6 <- names5[1:(nrow(PE.Var4)-1)]    #remove r(sps)
names6 <- names6[PE.Var4[1:(nrow(PE.Var4)-1),ncol(PE.Var4)] != 0]
use.colors <- c('darkblue','red','green','orange','yellow','purple','brown','cyan4','olivedrab4','hotpink1','lightblue')[1:(length(names6))]

#Plot it
plot(PE.Var4["V(A)",1:(ncol(PE.Var4)-1)],ylim=c(0,ymax),type='n',lwd=2,col='black',
     xlab='Generation Number',ylab='Variance or Covariance',bty='n',axes=TRUE)
lim <- par("usr")
labels.at2 <- seq(lim[1],lim[2],length.out=length(input)+1)

mtext("Change in Variance Across Generations - Does Not Include V(P)",line=6,cex=1.5)
mtext("Unstandardized Variance Components",line=5,cex=.7)
mtext(names5,side=3,outer=FALSE,line=4, cex=.7,at=labels.at[2:length(labels.at2)])
mtext(c("Start:",round(start2,2)),side=3,outer=FALSE,line=3, cex=.7,at=labels.at[1:length(labels.at2)])
mtext(c("End:",round(end2,2)),side=3,outer=FALSE,line=2, cex=.7,at=labels.at[1:length(labels.at2)])
mtext(c("Data:",round(dat2,2)),side=3,outer=FALSE,line=1, cex=.7,at=labels.at[1:length(labels.at2)])

legend(x=c(lim[1],(lim[2]-.1*(lim[2]-lim[1]))),
       y=c(lim[4],(lim[4]-.15*(lim[4]-lim[3]))),
       legend=names6,lty=1,col=use.colors,lwd=2,ncol=min(5,length(names6)),bg='snow2',yjust=1,text.width=1)

for (i in 1:length(names6)){
lines(PE.Var4[names6[i],1:(ncol(PE.Var4)-1)],col=use.colors[i],lwd=2)}
###############################################################################







############################################################################
#5 GRAPH CHANGES IN VA DUE TO AM

ex <- expectedVA(z=track.changes,n=PARAMS$number.genes)
Plot.expectedVA(y=PE.Var3, expected=ex,input=input)
############################################################################




})




########################
#END GRAPHICS SCRIPT

par(op)

dev.off()

}
########################








